/*
 * Copyright 2022 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef MISC_H
#define MISC_H

#include <QVariant>
#include <QDBusPendingReply>
#include <QDBusInterface>
#include <QThread>
#include <QApplication>

class Misc
{
public:
    Misc();

    template <typename T>
    static QDBusPendingReply<T> DbusAsyncCall(const QString &service,
                                              const QString &path,
                                              const QString &interface,
                                              const QDBusConnection &connection,
                                              const QString &method)
    {
        QDBusInterface dbusInterface(service, path, interface, connection);
        QDBusPendingReply<T> reply = dbusInterface.asyncCall(method);
        while (!reply.isFinished()) {
            qApp->processEvents();
            QThread::msleep(1);
        }
        return reply;
    }

    template <typename T>
    static QDBusPendingReply<T> DbusAsyncCallWithArgs(const QString &service,
                                                      const QString &path,
                                                      const QString &interface,
                                                      const QDBusConnection &connection,
                                                      const QString &method,
                                                      const QList<QVariant> &argList)
    {
        QDBusInterface dbusInterface(service, path, interface, connection);
        QDBusPendingReply<T> reply = dbusInterface.asyncCallWithArgumentList(method, argList);
        while (!reply.isFinished()) {
            qApp->processEvents();
            QThread::msleep(1);
        }
        return reply;
    }
};

#endif // MISC_H
